# coding: utf-8
u'''
Основной грид реестра циклов по курсам.

'''

from java.io import File, FileInputStream, FileOutputStream
from java.sql import Timestamp
from java.text import SimpleDateFormat
import json
import os

from com.jayway.jsonpath import JsonPath
from common.sysfunctions import toHexForXml, getGridHeight
from edu._edu_orm import course_cycleCursor, courseCursor
from gridsettings._gridsettings_orm import grid_fieldCursor, \
    user_grid_fieldCursor
from gridsettings.functions.gridSettings import toolbar_button, get_column_meta, \
    get_column_header, get_column_data, toolbar_button_default, filter_button, filter_reset_button
from medstaff._medstaff_orm import student_certificateCursor, \
    vw_certificateCursor, certificate_addressCursor
from nci._nci_orm import edu_organizationCursor, status_listCursor
from nci.functions.persons import getPersonIdBySid
from ru.curs.celesta.showcase.utils import XMLJSONConverter
from security.functions import userHasPermission
from umk._umk_orm import specialityCursor

from medstaff.reports.print_jasper import printToPdf


try:
    from ru.curs.showcase.core.jython import JythonDownloadResult
except:
    from ru.curs.celesta.showcase import JythonDownloadResult

try:
    from ru.curs.showcase.core.jython import JythonDTO
except:
    from ru.curs.celesta.showcase import JythonDTO


def gridData(context, main, add=None, filterinfo=None, session=None, elementId=None,
             sortColumnList=None, firstrecord=None, pagesize=None):
    session = json.loads(session)
    sid = JsonPath.read(session, "$.sessioncontext.sid")
    vw_certificate = vw_certificateCursor(context)
    grid_field = grid_fieldCursor(context)
    user_grid_field = user_grid_fieldCursor(context)
    person_id = getPersonIdBySid(context, sid)
    if elementId != 'certificationMethodGrid':
        vw_certificate.setRange('person_id', person_id)
        # Статус - на проверке
#         vw_certificate.setRange('status_id', 2)

    _header = {'uid': ['~~id','id'],
               'properties': [u'properties', 'properties']}    
    
    _header.update(get_column_header(grid_field,user_grid_field,elementId,sid))
    if elementId == "certificationRequestGrid":
        _header['cancel_request'] = [u'Отмена заявки', 'cancel_request']
      
    for column in _header:
        _header[column].append(toHexForXml(_header[column][0]))     

    if elementId == 'certificationMethodGrid' and context.getData().get('certificateFilter',''):
        ext_filter(context.getData()['certificateFilter'],vw_certificate,context)
    
    vw_certificate.limit(firstrecord - 1, pagesize)
    vw_certificate.orderBy('certificate_number DESC')
    grid_data = {'records': {'rec': []}}    
#     sdf = SimpleDateFormat("dd.MM.yyyy")    
    for rec in vw_certificate.iterate():
        row = {}
        row[_header['uid'][2]] = rec.uid
        
        add_data = {}
       
#         add_data = {'course_name':{'div':{'ins':{'font':{"@style": "cursor: pointer",
#                                                       '@color':'blue',
#                                                       "@title": u"Просмотреть подробную информацию о программе",
#                                                      '#text': rec.course_name}}}}}        
        
        get_column_data(_header, grid_field, user_grid_field, elementId, sid, vw_certificate, row, **add_data)     
        if elementId == "certificationRequestGrid":
            if rec.status_id==2:
                row[_header['cancel_request'][2]] =  {'div':{'ins':{'font':{"@style": "cursor: pointer",
                                                          '@color':'blue',
                                                          "@title": u"Отозвать заявку на программу",
                                                         '#text': u"Отозвать заявку"}}}}
        
        row[_header['properties'][1]] =  {'event': []}   
        row[_header['properties'][1]]['event'].append({
            '@name': 'row_single_click',
            'action': {
                '#sorted': [
                    {'main_context': 'current'},
                    {'datapanel': {
                        '@type': 'current',
                        '@tab': 'current'}}]}})
        
        if 'cancel_request' in _header and rec.status_id==2:
            row[_header['properties'][1]]['event'].append( {
                '@name': 'cell_single_click',
                '@column': _header['cancel_request'][0],
                'action': {
                    '@show_in': 'MODAL_WINDOW',
                    '#sorted': [
                        {'main_context': 'current'},
                        {'modalwindow': {
                            '@caption': u'Отмена заявки',
                            '@height': '140',
                            '@width': '400'}},
                            {'datapanel': {
                                '@type': 'current',
                                '@tab': 'current',
                                'element': {
                                    '@id': 'cancelRequestCard',
                                    'add_context': rec.uid}}}]}})
        
        
        grid_data['records']['rec'].append(row)

    return JythonDTO(XMLJSONConverter.jsonToXml(json.dumps(grid_data)), None)


def gridMeta(context, main=None, add=None, filterinfo=None, session=None, elementId=None):
    sid = JsonPath.read(session, "$.sessioncontext.sid")
    session = json.loads(session)
    # Заголовки
    vw_certificate = vw_certificateCursor(context)
    person_id = getPersonIdBySid(context, sid)
    if elementId != 'certificationMethodGrid':
        vw_certificate.setRange('person_id', person_id)
        # Статус - на проверке
#         vw_certificate.setRange('status_id', 2)

    numOfGrids = 1
    delta = 180
    properties_name = 'default_excel.properties'

#     filter_string = ''
#     if elementId == 'certificationMethodGrid' and context.getData().get('certificateFilter',''):
#         filter_string = ext_filter(context.getData()['certificateFilter'],vw_certificate,context)

    grid_settings = {}
    grid_settings['gridsettings'] = {
#         'labels': {
#             'header': header + ' <br>' + filter_string
#         },
        'columns': {
            'col': get_column_meta(context,elementId,sid)},
        'properties': {
            '@pagesize': 50,
            '@gridWidth': '100%',
            '@gridHeight': getGridHeight(session, numberOfGrids=numOfGrids, gridHeaderHeight=55, delta=delta),
            '@totalCount': vw_certificate.count(),
            '@profile': properties_name}}
    
    if elementId == "certificationRequestGrid":
        grid_settings['gridsettings']["columns"]["col"].append({"@id": u"Отмена заявки", "@width":"100px", "@horAlign": "CENTER"})

    return JythonDTO(None, XMLJSONConverter.jsonToXml(json.dumps(grid_settings)))


def gridToolBar(context, main=None, add=None, filterinfo=None, session=None, elementId=None):

    session_json = json.loads(session)['sessioncontext']
    sid = session_json['sid']
    grid_context = session_json['related']['gridContext']
    userdata = session_json['userdata']

    # редактирование
    edit_style = "false" if 'currentRecordId' in grid_context else "true"

    change_status_style = edit_style


    grid_toolbar = {'gridtoolbar': {
                        '#sorted': [
                            toolbar_button(context, sid, elementId),
                            toolbar_button_default(context, sid, elementId)]}}
    
    if userHasPermission(context,sid,'certificateEdit') and userdata != 'medstaff':
        grid_toolbar['gridtoolbar']['#sorted'].extend([
                    filter_button('certificateFilter',700, 700),
                    filter_reset_button('certificationMethodGrid','certificateFilter'),
                    {'item': {
                        '@text': u'Редактировать',
                        '@hint': u'Редактировать заявку',
                        '@disable': edit_style,
                        'action': {
                            '@show_in': 'MODAL_WINDOW',
                            '#sorted': [
                                {'main_context': 'current'},
                                {'modalwindow': {
                                    '@caption': u'Редактировать',
                                    '@height': '680',
                                    '@width': '660'}},
                                {'datapanel': {
                                    '@type': 'current',
                                    '@tab': 'current',
                                    'element': {
                                        '@id': 'certificateEditCard',
                                        'add_context': 'edit'}}}]}}},
                    {'item': {
                        '@text': u'Изменить статус',
                        '@hint': u'Изменить статус',
                        '@disable': change_status_style,
                        'action': {
                            '@show_in': 'MODAL_WINDOW',
                            '#sorted': [
                                {'main_context': 'current'},
                                {'modalwindow': {
                                    '@caption': 'Изменить статус',
                                    '@height': '300',
                                    '@width': '450'}},
                                {'datapanel': {
                                    '@type': 'current',
                                    '@tab': 'current',
                                    'element': {
                                        '@id': 'changeCertificateStatusCard',
                                        'add_context': ''}}}]}}}])
    
    return XMLJSONConverter.jsonToXml(json.dumps(grid_toolbar))


def ext_filter(session,vw_certificate,context):
    edu_organization = edu_organizationCursor(context)
    status_list = status_listCursor(context)
    sdf = SimpleDateFormat('dd.MM.yyyy')
    filter_string = u"Фильтр: "
    filter_list = []
    whereclause = '1 = 1'
    data = session['schema']['record']
    name = data['name']
    surname = data['surname']
    patronymic = data['patronymic']
    snils = data['snils']
    fromDate = data['start_date']
    toDate = data['end_date']
    speciality_id = data['speciality_id']
    speciality_name = data['speciality_name']
    edu_org_id = data['edu_organization_id']
    program_id = data['program_id']
    program_name = data['program_name']
    status_id = data['status_id']
    vw_certificate.setFilter('name',"@%%'%s'%%"%(name))
    if name:
        filter_list.append(u'имя содержит %s'%(name))
    vw_certificate.setFilter('surname',"@%%'%s'%%"%(surname))
    if surname:
        filter_list.append(u'фамилия содержит %s'%(surname))
    vw_certificate.setFilter('patronymic',"@%%'%s'%%"%(patronymic))
    if patronymic:
        filter_list.append(u'отчество содержит %s'%(patronymic))
    if snils:
        filter_list.append(u'СНИЛС содержит %s'%(snils))
    vw_certificate.setFilter('snils',"@%%'%s'%%"%(snils))
    if fromDate != '':
        fromDate = Timestamp.valueOf(u'%s 00:00:00.0' % fromDate)
    if toDate != '':
        toDate = Timestamp.valueOf(u'%s 23:59:59.0' % toDate)
    if fromDate != '' and toDate != '':
        filter_list.append(u'дата начала с %s по %s'%(sdf.format(fromDate),sdf.format(toDate)))
        vw_certificate.setRange('start_date',fromDate,toDate)
    elif fromDate != '' and toDate == '':
        filter_list.append(u'дата начала с %s'%(sdf.format(fromDate)))
        whereclause += " and start_date >= '%s'"%(fromDate)
#             vw_schedulelesson.setFilter('timeStart','"20131124"')
#             vw_schedulelesson.setFilter('timeStart',"('%s'..)" % (fromDate))
    elif fromDate == '' and toDate != '':
        filter_list.append(u'дата начала  по %s'%(sdf.format(toDate)))
        whereclause += " and start_date <= '%s'"%(toDate)
    if speciality_id:
        filter_list.append(u'специальность: %s'%(speciality_name))
        vw_certificate.setRange('spec_id',speciality_id)
    if edu_org_id and edu_org_id != '-':
        edu_organization.get(edu_org_id)
        filter_list.append(u'образовательная организация: %s'%(edu_organization.abbreviation))
        vw_certificate.setRange('edu_org_id',edu_org_id)
    if program_id:
        filter_list.append(u'название программы содержит %s'%(program_name))
        vw_certificate.setRange('course_id', program_id)
    if status_id:
        status_list.get(status_id)
        filter_list.append(u'статус равен %s'%(status_list.name))
        vw_certificate.setRange('status_id', status_id)
    #             vw_schedulelesson.setFilter('timeStart',"(..'%s')" % (toDate))
#     raise Exception(whereclause)
    filter_string = '<b>' + filter_string + ','.join(filter_list) +'</b>'
    vw_certificate.setComplexFilter(whereclause)
    return filter_string    


def certificateDownload(context, main=None, add=None, filterinfo=None, session=None, elementId=None, record_id=None):
    session = json.loads(session)['sessioncontext']
    student_certificate = student_certificateCursor(context)
    course = courseCursor(context)
    course_cycle = course_cycleCursor(context)
    edu_organization = edu_organizationCursor(context)
    speciality = specialityCursor(context)
    sdf = SimpleDateFormat('dd.MM.yyyy')
    certificate_address = certificate_addressCursor(context)
    student_certificate.get(record_id)
    
    course_cycle.get(student_certificate.cycle_id)
    course.get(course_cycle.course_id)
    speciality.get(course.speciality_id)
    edu_organization.get(course.edu_organization_id)
    certificate_address.get(record_id)    
    
    data = {'data': {
             'request_number':student_certificate.certificate_number,
             'when_create':sdf.format(student_certificate.issue_date),
             'cycle_startdate':sdf.format(course_cycle.start_date) if course_cycle.start_date else '',
             'cycle_enddate':sdf.format(course_cycle.end_date) if course_cycle.end_date else '',            
             'accred_cycle_status_id':'',
             'accred_cycle_status_name':'',
             'accred_cycle_status_comment':'',
             'dpp_name':course.name,
             'dpp_hours':course.hours,
             'dpp_price':course.edu_cost if course.edu_cost else '',
             'speciality_name':speciality.name,
             'specialization_name':'',
             'person': {
                'patronymic':student_certificate.patronymic,
                'name':student_certificate.name,
                'surname':student_certificate.surname,
                'snils': student_certificate.snils,
                'fio':'%s %s. %s.'%(student_certificate.surname, student_certificate.name[0], student_certificate.patronymic[0]),
                'full_name': '%s %s %s'%(student_certificate.surname,student_certificate.name,student_certificate.patronymic),
                'post':student_certificate.position if student_certificate.position else ''},
             'edu_org': {
                'abbreviation':'',
                'dpo_department':'',
                'responsible_person_post':'',
                'responsible_person_name':'',
                'address':'',
                'name':edu_organization.name,
                'shortname': edu_organization.shortname},
             'person_employer': {
                 'name':student_certificate.organization_name,
                 'address':certificate_address.address},
             'imgPath':os.path.join(os.path.dirname(os.path.abspath(__file__)), 'fon6.png')}}
    
    dataXml = XMLJSONConverter.jsonToXml(json.dumps(data))                    
#     pdfstreams = [getOnePdfStream(dataXml,
#                   os.path.join(os.path.dirname(os.path.abspath(__file__)), 'sertFrameSS1.jasper'))]
    report = FileOutputStream('cert.pdf')
    printToPdf(dataXml, File(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'zayavka.jasper')), report)
#     concatenatePdf(pdfstreams, report)
    report.close()
    report = File('cert.pdf')
    return JythonDownloadResult(FileInputStream(report), u'Сертификат ПК' + '.pdf')
