# coding: utf-8
'''
Created on 04.08.2014
Карточка ФЛ
@author: v.popov
'''
import hashlib
from java.text import SimpleDateFormat
import json
import re

from com.jayway.jsonpath import JsonPath
from common.numbersseries.getNextNo import getNextNoOfSeries
from nci._nci_orm import personCursor
from nci.functions.persons import checkSnilsUnique, getPersonIdBySid
from ru.curs.celesta.showcase.utils import XMLJSONConverter
from security._security_orm import subjectsCursor, loginsCursor
from security.functions import userHasPermission


try:
    from ru.curs.showcase.core.jython import JythonDTO
except:
    from ru.curs.celesta.showcase import JythonDTO

def cardData(context, main=None, add=None, filterinfo=None, session=None, elementId=None):
    u'''Карточка ФЛ'''
    person = personCursor(context)
    sid = JsonPath.read(session, "$.sessioncontext.sid")
    # определяем идентификатор ФЛ по sid
    person_id = getPersonIdBySid(context, sid)
    person.get(person_id)

    xformsdata = {"schema":
                      {"@xmlns":"",
                        "record":
                            {"uid": person_id
                           , "surname": person.surname
                           , "name": person.name
                           , "birthDate": SimpleDateFormat("yyyy-MM-dd").format(person.birthDate) if person.birthDate else ""
                           , "patronymic": person.patronymic
                           , "email": person.email
                           , "phone": person.phone
                           , "phoneExtra": person.phoneExtra
                           , "snils": person.snils
                           , "password": ""
                           , "passwordRepeat": ""
                           , "sex": person.sex
                           , "personal_data_agree": "true" if person.personal_data_agree or person.personal_data_agree is None else "false"
                           , "readonly": "true"
                            }
                        }
                      }

    xformssettings = {"properties":{"event":[{"@name":"single_click",
                                              "@linkId": "1",
                                              "action":{
                                                  "@keep_user_settings": "true",
                                                  '#sorted': [
                                                    {"main_context": "current"},
                                                    {"datapanel": {
                                                        "@type": "current",
                                                                  "@tab": "current",
                                                                  "element": {"@id":"personalDataCard",
                                                                              "add_context": ""
                                                                              }
                                                                  }}]
                                              }
                                            }]
                                    }
                      }

    return JythonDTO(XMLJSONConverter.jsonToXml(json.dumps(xformsdata)),
                     XMLJSONConverter.jsonToXml(json.dumps(xformssettings)))

def cardSave(context, main=None, add=None, filterinfo=None, session=None,
             elementId=None, xformsdata=None):
    u'''Сохранение данных о ФЛ. '''

    person = personCursor(context)

    data_dict = json.loads(xformsdata)["schema"]["record"]

    if not data_dict["uid"]:
        # Добавление новой записи в person
        person.insert()
    else:
        # Редактирование уже существующей
        person.get(data_dict["uid"])
    person.surname = data_dict["surname"]
    person.name = data_dict["name"]
    person.patronymic = data_dict["patronymic"]
    person.snils = data_dict["snils"]
    person.sex = data_dict["sex"]
    person.email = data_dict["email"]
#     if data_dict["phone"] and not checkPhoneNumber(data_dict["phone"]) or \
#             data_dict["phoneExtra"] and checkPhoneNumber(data_dict["phoneExtra"]):
#         context.error(u"Проверьте формат номера телефона")
    person.phone = data_dict["phone"]
    person.phoneExtra = data_dict["phoneExtra"]
    person.birthDate = SimpleDateFormat("yyyy-MM-dd").parse(data_dict["birthDate"]) if data_dict["birthDate"] else None
    person.personal_data_agree = data_dict["personal_data_agree"] == "true"
    person.fullname = u'%s %s %s' % (person.surname or '',
                                     person.name or '',
                                     person.patronymic or '')
    if data_dict["password"]:
        if data_dict["password"] != data_dict["passwordRepeat"]:    # пароли должны совпадать
            context.error(u"Введенные пароли не совпадают!")
        logins = loginsCursor(context)
        logins.get(JsonPath.read(session, "$.sessioncontext.username"))
        pass_hash = hashlib.sha1()    # Объект типа hash
        pass_hash.update(data_dict["password"])    # Записываем в него текущий пароль из карточки
        logins.password = pass_hash.hexdigest()
        logins.update()
        context.message(u"Пароль изменен")
    if not data_dict["uid"]:
        # Добавление новой записи в person
        person.uid = getNextNoOfSeries(context, 'persons')
        person.insert()
        context.commit()
        # вставляем связь пользователя и ФЛ
        subjects = subjectsCursor(context)
        subjects.sid = data_dict["sid"]
        subjects.name = person.fullname
        subjects.employeeId = person.uid
        if not subjects.tryInsert():
            subjects.update()
    else:
        # Редактирование уже существующей
        person.update()

def checkPhoneNumber(phone_number):
    u"""Начинается с плюса"""
    pattern = re.compile("^\+(?:[0-9] ?\-?){1,4}[0-9](?:[0-9] ?\-?){6,14}[0-9]$", re.IGNORECASE)
    return pattern.match(phone_number) is not None

